SystemOrganization addCategory: #'Seaside-Tests-JSON'!

!GRPackage class methodsFor: '*seaside-tests-json' stamp: 'pmm 2/18/2012 16:34'!
seasideTestsJson
	^ self new
		name: 'Seaside-Tests-JSON';
		description: 'Unit tests for the Seaside-JSON-Core package.';
		addDependency: 'Seaside-JSON-Core';
		url: #seasideUrl;
		yourself! !

TestCase subclass: #WAJsonParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-JSON'!

!WAJsonParserTest methodsFor: 'utilities' stamp: 'pmm 5/6/2012 16:23'!
assertInvalid: aString
	self should: [ self parse: aString ] raise: WAJsonSyntaxError! !

!WAJsonParserTest methodsFor: 'utilities' stamp: 'pmm 5/6/2012 16:23'!
parse: aString
	^ WAJsonParser parse: aString! !

!WAJsonParserTest methodsFor: 'tests-array' stamp: 'pmm 9/1/2012 16:28'!
testBigArray
	| result |
	result := self parse: ' [ "a" , "b" ] '.
	self assert: result isArray.
	self assert: result size = 2.
	self assert: result first = 'a'.
	self assert: result second = 'b'! !

!WAJsonParserTest methodsFor: 'tests-objects' stamp: 'pmm 9/1/2012 16:29'!
testBigObject
	| result |
	result := self parse: ' { "a" : 1 , "b" : 2 } '.
	self assert: result isDictionary.
	self assert: result size = 2.
	self assert: (result at: 'a') = 1.
	self assert: (result at: 'b') = 2! !

!WAJsonParserTest methodsFor: 'tests-array' stamp: 'pmm 12/15/2008 19:50'!
testEmptyArray
	| result |
	result := self parse: '[]'.
	self assert: result isArray.
	self assert: result isEmpty! !

!WAJsonParserTest methodsFor: 'tests-objects' stamp: 'pmm 12/15/2008 19:58'!
testEmptyObject
	| result |
	result := self parse: '{}'.
	self assert: result isDictionary.
	self assert: result isEmpty! !

!WAJsonParserTest methodsFor: 'tests-real' stamp: 'lr 6/29/2009 14:37'!
testExplorerEvent
	| result |
	result := self parse: '{"recordset": null, "type": "change", "fromElement": null, "toElement": null, "altLeft": false, "keyCode": 0, "repeat": false, "reason": 0, "behaviorCookie": 0, "contentOverflow": false, "behaviorPart": 0, "dataTransfer": null, "ctrlKey": false, "shiftLeft": false, "dataFld": "", "qualifier": "", "wheelDelta": 0, "bookmarks": null, "button": 0, "srcFilter": null, "nextPage": "", "cancelBubble": false, "x": 89, "y": 502, "screenX": 231, "screenY": 1694, "srcUrn": "", "boundElements": {"length": 0}, "clientX": 89, "clientY": 502, "propertyName": "", "shiftKey": false, "ctrlLeft": false, "offsetX": 25, "offsetY": 2, "altKey": false}'.
	self assert: (result at: 'type') = 'change'.
	self assert: (result at: 'offsetY') = 2.
	self deny: (result at: 'altKey').
	self assert: (result at: 'dataTransfer') isNil.
	self assert: (result at: 'nextPage') = ''! !

!WAJsonParserTest methodsFor: 'tests-literals' stamp: 'lr 6/25/2009 23:23'!
testFalse
	self deny: (self parse: 'false')! !

!WAJsonParserTest methodsFor: 'tests-real' stamp: 'lr 6/29/2009 14:36'!
testFirefoxEvent
	| result |
	result := self parse: '{"type": "change", "eventPhase": 2, "bubbles": true, "cancelable": true, "timeStamp": 0, "CAPTURING_PHASE": 1, "AT_TARGET": 2, "BUBBLING_PHASE": 3, "isTrusted": true, "MOUSEDOWN": 1, "MOUSEUP": 2, "MOUSEOVER": 4, "MOUSEOUT": 8, "MOUSEMOVE": 16, "MOUSEDRAG": 32, "CLICK": 64, "DBLCLICK": 128, "KEYDOWN": 256, "KEYUP": 512, "KEYPRESS": 1024, "DRAGDROP": 2048, "FOCUS": 4096, "BLUR": 8192, "SELECT": 16384, "CHANGE": 32768, "RESET": 65536, "SUBMIT": 131072, "SCROLL": 262144, "LOAD": 524288, "UNLOAD": 1048576, "XFER_DONE": 2097152, "ABORT": 4194304, "ERROR": 8388608, "LOCATE": 16777216, "MOVE": 33554432, "RESIZE": 67108864, "FORWARD": 134217728, "HELP": 268435456, "BACK": 536870912, "TEXT": 1073741824, "ALT_MASK": 1, "CONTROL_MASK": 2, "SHIFT_MASK": 4, "META_MASK": 8}'.
	self assert: (result at: 'type') = 'change'.
	self assert: (result at: 'eventPhase') = 2.
	self assert: (result at: 'bubbles').
	self assert: (result at: 'cancelable').
	self assert: (result at: 'BACK') = 536870912! !

!WAJsonParserTest methodsFor: 'tests-invalid' stamp: 'lr 6/25/2009 23:34'!
testInvalidArray
	self assertInvalid: '['.
	self assertInvalid: '[1'.
	self assertInvalid: '[1,'.
	self assertInvalid: '[1,]'.
	self assertInvalid: '[1 2]'.
	self assertInvalid: '[]]'! !

!WAJsonParserTest methodsFor: 'tests-invalid' stamp: 'lr 6/25/2009 23:31'!
testInvalidFalse
	self assertInvalid: 'fa'.
	self assertInvalid: 'falsely'.
	self assertInvalid: 'fabulous'! !

!WAJsonParserTest methodsFor: 'tests-invalid' stamp: 'lr 6/25/2009 23:31'!
testInvalidNull
	self assertInvalid: 'nu'.
	self assertInvalid: 'nuclear'.
	self assertInvalid: 'nullified'! !

!WAJsonParserTest methodsFor: 'tests-invalid' stamp: 'lr 6/25/2009 23:34'!
testInvalidObject
	self assertInvalid: '{'.
	self assertInvalid: '{"a"'.
	self assertInvalid: '{"a":'.
	self assertInvalid: '{"a":"b"'.
	self assertInvalid: '{"a":"b",'.
	self assertInvalid: '{"a"}'.
	self assertInvalid: '{"a":}'.
	self assertInvalid: '{"a":"b",}'.
	self assertInvalid: '{}}'! !

!WAJsonParserTest methodsFor: 'tests-invalid' stamp: 'lr 6/25/2009 23:33'!
testInvalidString
	self assertInvalid: '"'.
	self assertInvalid: '"a'.
	self assertInvalid: '"\"'.
	self assertInvalid: '"\a"'.
	self assertInvalid: '"\u"'.
	self assertInvalid: '"\u1"'.
	self assertInvalid: '"\u12"'.
	self assertInvalid: '"\u123"'.
	self assertInvalid: '"\u123x"'! !

!WAJsonParserTest methodsFor: 'tests-invalid' stamp: 'lr 6/25/2009 23:30'!
testInvalidTrue
	self assertInvalid: 'tr'.
	self assertInvalid: 'trace'.
	self assertInvalid: 'truest'! !

!WAJsonParserTest methodsFor: 'tests-array' stamp: 'pmm 9/1/2012 16:29'!
testNestedArray
	| result |
	result := self parse: '[[2]]'.
	self assert: result isArray.
	self assert: result size = 1.
	result := result first.
	self assert: result isArray.
	self assert: result size = 1.
	self assert: result first = 2! !

!WAJsonParserTest methodsFor: 'tests-objects' stamp: 'pmm 9/1/2012 16:29'!
testNestedObject
	| result |
	result := self parse: '{"object":{"1":"2"}} '.
	self assert: result isDictionary.
	self assert: result size = 1.
	result := result at: 'object'.
	self assert: result isDictionary.
	self assert: result size = 1.
	self assert: (result at: '1') = '2'! !

!WAJsonParserTest methodsFor: 'tests-literals' stamp: 'lr 6/25/2009 23:23'!
testNull
	self assert: (self parse: 'null') isNil! !

!WAJsonParserTest methodsFor: 'tests-literals' stamp: 'mlucas-smith 6/29/2009 18:20'!
testNumberFloat
	self assert: (self parse: '0.0') = 0.0.
	self assert: (self parse: '0.12') = 0.12.
	self assert: (self parse: '-0.12') = -0.12.
	self assert: (self parse: '12.34') = 12.34.
	self assert: (self parse: '-12.34') = -12.34.
	self assert: (self parse: '1.2e0') = 1.2.
	self assert: (self parse: '1.2e+1') = 12.0.
	self assert: (self parse: '1.2e-1') = 1.2e-1! !

!WAJsonParserTest methodsFor: 'tests-literals' stamp: 'lr 6/25/2009 23:08'!
testNumberInteger
	self assert: (self parse: '0') = 0.
	self assert: (self parse: '1') = 1.
	self assert: (self parse: '-1') = -1.
	self assert: (self parse: '12') = 12.
	self assert: (self parse: '-12') = -12.
	self assert: (self parse: '1e2') = 100.
	self assert: (self parse: '1e+2') = 100! !

!WAJsonParserTest methodsFor: 'tests-real' stamp: 'lr 6/29/2009 14:36'!
testSafariEvent
	| result |
	result := self parse: '{"returnValue": true, "timeStamp": 1226697417289, "eventPhase": 2, "type": "change", "cancelable": false, "bubbles": true, "cancelBubble": false, "MOUSEOUT": 8, "FOCUS": 4096, "CHANGE": 32768, "MOUSEMOVE": 16, "AT_TARGET": 2, "SELECT": 16384, "BLUR": 8192, "KEYUP": 512, "MOUSEDOWN": 1, "MOUSEDRAG": 32, "BUBBLING_PHASE": 3, "MOUSEUP": 2, "CAPTURING_PHASE": 1, "MOUSEOVER": 4, "CLICK": 64, "DBLCLICK": 128, "KEYDOWN": 256, "KEYPRESS": 1024, "DRAGDROP": 2048}'.
	self assert: (result at: 'type') = 'change'.
	self assert: (result at: 'eventPhase') = 2.
	self assert: (result at: 'bubbles').
	self deny: (result at: 'cancelBubble').
	self deny: (result at: 'cancelable').
	self assert: (result at: 'BLUR') = 8192! !

!WAJsonParserTest methodsFor: 'tests-array' stamp: 'pmm 9/1/2012 16:29'!
testSmallArray
	| result |
	result := self parse: '["a"]'.
	self assert: result isArray.
	self assert: result size = 1.
	self assert: result first = 'a'! !

!WAJsonParserTest methodsFor: 'tests-objects' stamp: 'pmm 12/15/2008 19:59'!
testSmallObject
	| result |
	result := self parse: '{"a":1}'.
	self assert: result isDictionary.
	self assert: result size = 1.
	self assert: (result at: 'a') = 1! !

!WAJsonParserTest methodsFor: 'tests-literals' stamp: 'lr 6/25/2009 19:58'!
testString
	self assert: (self parse: '""') = ''.
	self assert: (self parse: '"foo"') = 'foo'.
	self assert: (self parse: '"foo bar"') = 'foo bar'! !

!WAJsonParserTest methodsFor: 'tests-literals' stamp: 'lr 7/16/2009 08:50'!
testStringEscaped
	self assert: (self parse: '"\""') = '"'.
	self assert: (self parse: '"\\"') = '\'.
	self assert: (self parse: '"\/"') = '/'.
	self assert: (self parse: '"\b"') = (String with: Character backspace).
	self assert: (self parse: '"\f"') = (String with: Character newPage).
	self assert: (self parse: '"\n"') = (String with: Character lf).
	self assert: (self parse: '"\r"') = (String with: Character cr).
	self assert: (self parse: '"\t"') = (String with: Character tab).
	self assert: (self parse: '"\u20Ac"') = (String with: (Character codePoint: 16r20AC))! !

!WAJsonParserTest methodsFor: 'tests-literals' stamp: 'lr 6/25/2009 23:23'!
testTrue
	self assert: (self parse: 'true')! !

TestCase subclass: #WAJsonRenderingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-JSON'!

!WAJsonRenderingTest methodsFor: 'private' stamp: 'pmm 1/22/2012 16:24'!
assert: aBlock gives: aString
	| json |
	json := String streamContents: [ :stream |
		| context document canvas |
		document := WAJsonDocument on: stream.
		context := WARenderContext new
			actionUrl: WAUrl new;
			document: document;
			yourself.
		 canvas := WAJsonCanvas context: context.
		canvas render: aBlock.
		document close ].
	self
		assert: json = aString
		description:	'Expected: ' , aString printString , ', Actual: ', json printString
		resumable: true! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 14:26'!
testArray
	self
		assert: [ :json |
			json array: [
				json
					value: 1;
					value: '1';
					value: true;
					value: false;
					value: nil ] ]
		gives: '[1, "1", true, false, null]'.
	self
		assert: [ :json |
			json array: [
				json value: 1 ] ]
		gives: '[1]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 8/24/2011 12:03'!
testNestedArray
	self
		assert: [ :json |
			json array: [
				json
					value: 1;
					value: [
						json array: [
							json value: '1' ] ];
					value: 2 ] ]
		gives: '[1, ["1"], 2]'.
	self
		assert: [ :json |
			json array: [
				json array: [
					json value: 1 ] ] ] 
		gives: '[[1]]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 8/24/2011 12:06'!
testNestedObject
	self
		assert: [ :json |
			json object: [
				json
					key: 'a' value: 'b';
					key: '1' value: [
						json object: [
							json key: '2' value: 2 ] ];
					key: 'c' value: 'd' ] ]
		gives: '{"a": "b", "1": {"2": 2}, "c": "d"}'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 8/24/2011 13:58'!
testObject
	self
		assert: [ :json |
			json object: [
				json
					key: '1' value: '1';
					key: '2' value: 2 ] ]
		gives: '{"1": "1", "2": 2}'.
	self
		assert: [ :json |
			json object: [
				json key: '1' value: 1 ] ]
		gives: '{"1": 1}'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 6/24/2012 11:42'!
testSpecialFloats
	"Tests that float values that can't be represented as JSON objects are null
	http://code.google.com/p/seaside/issues/detail?id=730
	http://ecma-international.org/ecma-262/5.1/#sec-15.12.3"
	self
		assert: [ :json |
			json array: [
				json
					value: Float infinity;
					value: Float infinity * -1;
					value: Float nan ] ]
		gives: '[null, null, null]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 5/6/2012 14:51'!
testStringEscape
	self
		assert: [ :json |
			json array: [
				json value: '"\/' ] ]
		gives: '["\"\\/"]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 5/6/2012 14:59'!
testStringEscapeNamed
	| backSpace formFeed lineFeed carriageReturn horizontalTab |
	backSpace := Character codePoint: 8.
	horizontalTab := Character codePoint: 9.
	lineFeed := Character codePoint: 10.
	formFeed := Character codePoint: 12.
	carriageReturn := Character codePoint: 13.
	self
		assert: [ :json |
			json array: [
				json value: (String streamContents: [ :stream |
					stream
						nextPut: backSpace;
						nextPut: horizontalTab;
						nextPut: lineFeed;
						nextPut: formFeed;
						nextPut: carriageReturn ]) ] ]
		gives: '["\b\t\n\f\r"]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 5/6/2012 15:12'!
testStringEscapeUnicode
	| lineSeparator paragraphSeparator snowMan |
	lineSeparator := Character codePoint: 16r2028.
	paragraphSeparator := Character codePoint: 16r2028.
	paragraphSeparator := Character codePoint: 16r2028.
	snowMan := Character codePoint: 16r2603.
	self
		assert: [ :json |
			json array: [
				json value: (String streamContents: [ :stream |
					stream
						nextPut: lineSeparator;
						nextPut: paragraphSeparator;
						nextPut: snowMan ]) ] ]
		gives: '["', (String with: lineSeparator), (String with: paragraphSeparator), (String with: snowMan), '"]'! !

TestCase subclass: #WAJsonStreamTest
	instanceVariableNames: 'printer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-JSON'!

!WAJsonStreamTest methodsFor: 'asserting' stamp: 'pmm 2/18/2012 16:06'!
assert: anObject equals: aString
	self
		assert: anObject asJson = aString
		description: anObject asJson , ' expected to be ' , aString greaseString
		resumable: true! !

!WAJsonStreamTest methodsFor: 'asserting' stamp: 'pmm 5/1/2013 15:05'!
assert: anObject equalsIgnoreCase: aString
	self
		assert: anObject asJson asLowercase = aString asLowercase
		description: anObject asJson , ' expected to be ' , aString greaseString
		resumable: true! !

!WAJsonStreamTest methodsFor: 'printing' stamp: 'jf 1/21/2013 06:59'!
jsonOn: aRenderer
	printer value: aRenderer! !

!WAJsonStreamTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 16:06'!
testArray
	self assert: #(1 2 3) equals: '[1, 2, 3]'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testCharacter
	self assert: $a equals: '"a"'! !

!WAJsonStreamTest methodsFor: 'tests-custom' stamp: 'jf 1/21/2013 06:58'!
testCustomArray
	printer := [ :renderer | renderer render: #('aString1' ('aString2' 'aString3')) ].
	self assert: self equals: '["aString1", ["aString2", "aString3"]]'! !

!WAJsonStreamTest methodsFor: 'tests-custom' stamp: 'jf 1/21/2013 06:58'!
testCustomDictionary
	printer := [ :renderer | 
		renderer render: (GRSmallDictionary new
			at: 'foo' put: 'bar';
			at: 'zork' put: (GRSmallDictionary new
				at: 'foo' put: 'bar';
				yourself);
			yourself) ].
	self assert: self equals: '{"foo": "bar", "zork": {"foo": "bar"}}'! !

!WAJsonStreamTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 16:06'!
testDictionary
	self
		assert:
			((GRSmallDictionary new)
				at: 'foo' put: 1;
				at: 'bar' put: 2;
				at: 'zork' put: 3;
				yourself)
		equals: '{"foo": 1, "bar": 2, "zork": 3}'.
	self
		assert:
			((GRSmallDictionary new)
				at: '' put: 'empty';
				yourself)
		equals: '{"": "empty"}'.
	self
		assert:
			((Dictionary new)
				at: 1 put: 'not-a-string';
				yourself)
		equals: '{1: "not-a-string"}'.
	self
		assert:
			((Dictionary new)
				at: '1abc' put: 'not-an-id';
				yourself)
		equals: '{"1abc": "not-an-id"}'.
	self
		assert:
			((Dictionary new)
				at: 'abc1' put: 'totally sexy';
				yourself)
		equals: '{"abc1": "totally sexy"}'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testLiteral
	self assert: true equals: 'true'.
	self assert: false equals: 'false'.
	self assert: nil equals: 'null'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 9/1/2012 16:29'!
testNumber
	self assert: 123 equals: '123'.
	self assert: 123.4 equals: '123.4'! !

!WAJsonStreamTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 16:06'!
testObject
	self
		should: [ Object new asJson ]
		raise: Error! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testStringBasic
	self assert: 'abc' equals: '"abc"'.
	self assert: '123' equals: '"123"'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testStringEscaping
	self assert: '''' equals: '"''"'.
	self assert: '"' equals: '"\""'.
	self assert: '\' equals: '"\\"'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 9/1/2012 16:29'!
testStringMultiByte
	"regression test for
	Issue 217: MultibyteString JavascriptOn infinite recursion [GemStone]
	http://code.google.com/p/glassdb/issues/detail?id=217"
	| multibyteString |
	multibyteString := String with: (Character codePoint: 8364).
	self assert: multibyteString equals: '"', multibyteString, '"'
! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 5/1/2013 15:06'!
testStringSpecial
	self assert: (String with: Character cr) equals: '"\r"'.
	self assert: (String with: Character lf) equals: '"\n"'.
	self assert: (String with: Character tab) equals: '"\t"'.
	self assert: (String with: (Character codePoint: 8)) equals: '"\b"'.
	self assert: (String with: (Character codePoint: 12)) equals: '"\f"'.

	"http://tools.ietf.org/html/rfc4627#section-2.5"
	0 to: 16r001F do: [ :codePoint |
		(#(8 "backspace" 9 "tab" 10 "lf" 12 "form feed" 13 "cr") includes: codePoint) ifFalse: [
			self
				assert: (String with: (Character codePoint: codePoint))
				"case doesn't matter"
				equalsIgnoreCase: '"\u', ((codePoint printStringBase: 16) padded: #left to: 4 with: $0), '"'  ] ]! !
